CPU 386

%define _PROC16_
%include "segs.inc"
%include "proc.mac"

org (BOOTSEG << 4)
section .text
BITS 16

;**********************************************************************************************************

start:

;--- setup crap ---
    ;data segment = code segment
    push cs
    pop ds
    cld             ;just for fun

    ;80x25 16-color text
    mov ax,0003h
    int 10h
;------------------

    ;this is why it needs to be a 32-bit cpu
    ;but who'd have anything else?

    xor ebx,ebx
  .again:
    mov eax,0E820h
    mov edx,'PAMS'
    mov ecx,20
    mov di,buff
    push ds
    pop es
    int 15h

    cmp eax,'PAMS'
    jne .fail
    jc boot

    push ebx
    func showresult
    pop ebx
    test ebx,ebx
    jnz .again
    jmp boot

  .fail:
    func prmsg, feck
    jmp boot

;***************************************************************************************************

ALIGN 4
showresult proc
    ;printf("base: %08X%08X; len: %08X%08X; type: %08X\n", buff[00h],buff[04h], buff[08h],buff[0Ch], buff[0Ch])
    uses si
    mov si,buff
    func prmsg, res_1
    func prhex, word [si+06h]
    func prhex, word [si+04h]
    func prmsg, res__
    func prhex, word [si+02h]
    func prhex, word [si+00h]
    func prmsg, res_2
    func prhex, word [si+0Eh]
    func prhex, word [si+0Ch]
    func prmsg, res__
    func prhex, word [si+0Ah]
    func prhex, word [si+08h]
    func prmsg, res_3
    func prhex, word [si+12h]
    func prhex, word [si+10h]
    ret
endproc

ALIGN 4
prhex proc val
    ;16 bits = 4 hex digits
    mov dx,[arg(val)]
    mov cx,4
  .yay:
    rol dx,4
    mov ax,dx
    and ax,0fh
    cmp al,0Ah
    sbb al,69h
    das
    mov ah,0Eh
    int 10h
    dec cx
    jnz .yay
    ret
endproc

ALIGN 4
prmsg proc pString
    uses si
    mov si,[arg(pString)]
    mov ah,0Eh
  .lp:
    mov al,[si]
    test al,al
    jz .out
    int 10h
    inc si
    jmp .lp
  .out:
    ret
endproc

ALIGN 4
boot proc
    xor ax,ax
    int 16h     ;wait keypress
    int 19h     ;boot
    NO_RET
endproc

;***************************************************************************************************

;.data
ALIGN 4,db 0
feck    db "E820 System Memory Map function failed",0
res__   db ":",0
res_1   db "Base: ",0
res_2   db "; len: ",0
res_3   db "; type: ",0
nl      db 13,10,0

;***************************************************************************************************

section .bss

buff    resb 20

;***************************************************************************************************


;INT 15 - newer BIOSes - GET SYSTEM MEMORY MAP (32-bit)
;    EAX = 0000E820h
;    EDX = 534D4150h ('SMAP')
;    EBX = continuation value or 00000000h to start at beginning of map
;    ECX = size of buffer for result, in bytes (should be >= 20 bytes)
;    ES:DI -> buffer for result
;
;Return:
;    CF clear if successful
;      EAX = 534D4150h ('SMAP')
;      ES:DI buffer filled
;      EBX = next offset from which to copy or 00000000h if all done
;      ECX = actual length returned in bytes
;
;    CF set on error
;      AH = error code (86h) (see #00496 at INT 15/AH=80h)
;
;Notes:
;    originally introduced with the Phoenix BIOS v4.0, this function is
;    now supported by most newer BIOSes, since various versions of Windows
;    call it to find out about the system memory
;
;    a maximum of 20 bytes will be transferred at one time, even if ECX is
;    higher; some BIOSes (e.g. Award Modular BIOS v4.50PG) ignore the
;    value of ECX on entry, and always copy 20 bytes
;
;    if this function is not supported, an application should fall back
;    to AX=E801h, and then AH=88h
;
;    the BIOS is permitted to return a nonzero continuation value in EBX
;    and indicate that the end of the list has already been reached by
;    returning with CF set on the next iteration
;
;    this function will return base memory and ISA/PCI memory contiguous
;    with base memory as normal memory ranges; it will indicate
;    chipset-defined address holes which are not in use and motherboard
;    memory-mapped devices, and all occurrences of the system BIOS as
;    reserved; standard PC address ranges will not be reported
;
;Format of Phoenix BIOS system memory map address range descriptor:
;Offset  Size    Description
; 00h    QWORD   base address
; 08h    QWORD   length in bytes
; 10h    DWORD   type of address range
;
;Values for System Memory Map address type:
; 01h    memory, available to OS
; 02h    reserved, not available (e.g. system ROM, memory-mapped device)
; 03h    ACPI Reclaim Memory (usable by OS after reading ACPI tables)
; 04h    ACPI NVS Memory (OS is required to save this memory between NVS sessions)
; other  not defined yet -- treat as Reserved

;----------------------------------------------------------------------------------------------------------

;INT 15 - Phoenix BIOS v4.0 - GET MEMORY SIZE FOR >64M CONFIGURATIONS (16-bit)
;    AX = E801h
;
;Return:
;    CF clear if successful
;      AX = extended memory between 1M and 16M, in K (max 3C00h = 15MB)
;      BX = extended memory above 16M, in 64K blocks
;      CX = configured memory 1M to 16M, in K
;      DX = configured memory above 16M, in 64K blocks
;    CF set on error
;
;Notes:
;    supported by the A03 level (6/14/94) and later XPS P90 BIOSes, as well
;    as the Compaq Contura, 3/8/93 DESKPRO/i, and 7/26/93 LTE Lite 386 ROM BIOS
;
;    supported by AMI BIOSes dated 8/23/94 or later
;    on some systems, the BIOS returns AX=BX=0000h; in this case, use CX
;    and DX instead of AX and BX
;
;    this interface is used by Windows NT 3.1, OS/2 v2.11/2.20, and is
;    used as a fall-back by newer versions if AX=E820h is not supported

;----------------------------------------------------------------------------------------------------------

;INT 15 - SYSTEM - GET EXTENDED MEMORY SIZE (286+)
;    AH = 88h
;
;Return:
;    CF clear if successful
;      AX = number of contiguous KB starting at absolute address 100000h
;    CF set on error
;      AH = status
;         80h invalid command (PC,PCjr)
;         86h unsupported function (XT,PS30)
;
;Notes:
;    TSRs which wish to allocate extended memory to themselves often hook
;    this call, and return a reduced memory size. They are then free to
;    use the memory between the new and old sizes at will.
;
;    the standard BIOS only returns memory between 1MB and 16MB; use AH=C7h
;    for memory beyond 16MB
;
;    not all BIOSes correctly return the carry flag, making this call
;    unreliable unless one first checks whether it is supported through
;    a mechanism other than calling the function and testing CF
;
;    Due to applications not dealing with more than 24-bit descriptors
;    (286), Windows 3.0 has problems when this function reports more
;    than 15 MB. Some releases of HIMEM.SYS are therefore limited to use
;    only 15 MB, even when this function reports more.

;----------------------------------------------------------------------------------------------------------

;INT 15 - Phoenix BIOS v4.0 - GET BIG MEMORY SIZE
;    AH = 8Ah
;
;Return:
;    DX:AX = extended memory size in K

;----------------------------------------------------------------------------------------------------------

;INT 15 U - AMI PCI BIOS - GET EXTENDED MEMORY SIZE
;    AX = DA88h
;
;Return:
;    CF clear (successful)
;      AX = 0000h
;      CL:BX = extended memory size in KBytes
